

#ifndef EVE_MAIL_H
#define EVE_MAIL_H

enum MailStatusMask {
  mailStatusMaskRead = 1,
  mailStatusMaskReplied = 2,
  mailStatusMaskForwarded = 4,
  mailStatusMaskTrashed = 8,
  mailStatusMaskDraft = 16,
  mailStatusMaskAutomated = 32,
};

enum mailLabelMask {
  mailLabelInbox = 1,
  mailLabelSent = 2,
  mailLabelCorporation = 4,
  mailLabelAlliance = 8,
  mailLabelsSystem = mailLabelInbox + mailLabelSent + mailLabelCorporation + mailLabelAlliance,
};


#define mailingListBlocked 0
#define mailingListAllowed 1
#define mailingListMemberMuted 0
#define mailingListMemberDefault 1
#define mailingListMemberOperator 2
#define mailingListMemberOwner 3


#define mailMaxRecipients 50
#define mailMaxGroups 1
#define mailMaxSubjectSize 150
#define mailMaxBodySize 8000
#define mailMaxTaggedBodySize 10000
#define mailMaxLabelSize 40
#define mailMaxNumLabels 25
#define mailMaxPerPage 100
#define mailTrialAccountTimer 1
#define mailMaxMessagePerMinute 5
#define mailinglistMaxMembers 3000
#define mailinglistMaxMembersUpdated 1000
#define mailingListMaxNameSize 60


// Notification data, as Notifications are part of mail system.

enum NotificationSecLevelDecsriptions {
     SecDesc_n10 = -10,
     SecDesc_n9 = -9,
     SecDesc_n8 = -8,
     SecDesc_n7 = -7,
     SecDesc_n6 = -6,
     SecDesc_n5 = -5,
     SecDesc_n4 = -4,
     SecDesc_n3 = -3,
     SecDesc_n2 = -2,
     SecDesc_n1 = -1,
     SecDesc_0 = 0,
     SecDesc_1 = 1,
     SecDesc_2 = 2,
     SecDesc_3 = 3,
     SecDesc_4 = 4,
     SecDesc_5 = 5,
     SecDesc_6 = 6,
     SecDesc_7 = 7,
     SecDesc_8 = 8,
     SecDesc_9 = 9,
     SecDesc_10 = 10
};

namespace Notify {

    namespace Groups {
        enum {
            Unread = 0,
            Agents = 1,
            Bills = 2,
            Corp = 3,
            Misc = 4,
            Old = 5,
            Sov = 6,
            Structures = 7,
            War = 8,
            Contacts = 9
        };
    }

    namespace Types {
        // these are types for mail msgs.
        enum {
            OldLscMessages = 1,
            CharTermination = 2,
            CharMedal = 3,
            AllMaintenanceBill = 4,
            AllWarDeclared = 5,
            AllWarSurrender = 6,
            AllWarRetracted = 7,
            AllWarInvalidated = 8,
            CharBill = 9,
            CorpAllBill = 10,
            BillOutOfMoney = 11,
            BillPaidChar = 12,
            BillPaidCorpAll = 13,
            BountyClaim = 14,
            CloneActivation = 15,
            CorpAppNew = 16,
            CorpAppReject = 17,
            CorpAppAccept = 18,
            CorpTaxChange = 19,
            CorpNews = 20,
            CharLeftCorp = 21,
            CorpNewCEO = 22,
            CorpDividend = 23,
            CorpVote = 25,
            CorpVoteCEORevoked = 26,
            CorpWarDeclared = 27,
            CorpWarFightingLegal = 28,
            CorpWarSurrender = 29,
            CorpWarRetracted = 30,
            CorpWarInvalidated = 31,
            ContainerPassword = 32,
            Customs = 33,
            InsuranceFirstShip = 34,
            InsurancePayout = 35,
            InsuranceInvalidated = 36,
            SovAllClaimFail = 37,
            SovCorpClaimFail = 38,
            SovAllBillLate = 39,
            SovCorpBillLate = 40,
            SovAllClaimLost = 41,
            SovCorpClaimLost = 42,
            SovAllClaimAquired = 43,
            SovCorpClaimAquired = 44,
            AllAnchoring = 45,
            AllStructVulnerable = 46,
            AllStrucInvulnerable = 47,
            SovDisruptor = 48,
            CorpStructLost = 49,
            CorpOfficeExpiration = 50,
            CloneRevoked1 = 51,
            CloneMoved = 52,
            CloneRevoked2 = 53,
            InsuranceExpiration = 54,
            InsuranceIssued = 55,
            JumpCloneDeleted1 = 56,
            JumpCloneDeleted2 = 57,
            FWCorpJoin = 58,
            FWCorpLeave = 59,
            FWCorpKick = 60,
            FWCharKick = 61,
            FWCorpWarning = 62,
            FWCharWarning = 63,
            FWCharRankLoss = 64,
            FWCharRankGain = 65,
            AgentMove = 66,
            TransactionReversal = 67,
            Reimbursement = 68,
            LocateChar = 69,
            ResearchMissionAvailable = 70,
            MissionOfferExpiration = 71,
            MissionTimeout = 72,
            StoryLineMissionAvailable = 73,
            Tutorial = 74,
            TowerAlert = 75,
            TowerResourceAlert = 76,
            StationAggression1 = 77,
            StationStateChange = 78,
            StationConquer = 79,
            StationAggression2 = 80,
            FacWarCorpJoinRequest = 81,
            FacWarCorpLeaveRequest = 82,
            FacWarCorpJoinWithdraw = 83,
            FacWarCorpLeaveWithdraw = 84,
            CorpLiquidation = 85,
            SovereigntyTCUDamage = 86,
            SovereigntySBUDamage = 87,
            SovereigntyIHDamage = 88,
            ContactAdd = 89,
            ContactEdit = 90,
            IncursionCompleted = 91,
            CorpKicked = 92,
            OrbitalAttacked = 93,
            OrbitalReinforced = 94,
            OwnershipTransferred = 95,
            FWAllianceWarning = 96,
            FWAllianceKick = 97,

            // these are internal corp Notifications to allow switching for various events
            MarketOrder = 125,    // since this updates same corp group, is used for all market order updates
            WalletChange = 126,
            ItemUpdateStation = 127,
            ItemUpdateSystem = 128,
            FactoryJob = 129            // factory jobs now have config option to add event to calendar.  this updates corp members 
        };
    }
}

/*
securityLevelDescriptions = {-10: 'Notifications/SecurityStatus/SecurityDescription_-10',
 -9: 'Notifications/SecurityStatus/SecurityDescription_-9',
 -8: 'Notifications/SecurityStatus/SecurityDescription_-8',
 -7: 'Notifications/SecurityStatus/SecurityDescription_-7',
 -6: 'Notifications/SecurityStatus/SecurityDescription_-6',
 -5: 'Notifications/SecurityStatus/SecurityDescription_-5',
 -4: 'Notifications/SecurityStatus/SecurityDescription_-4',
 -3: 'Notifications/SecurityStatus/SecurityDescription_-3',
 -2: 'Notifications/SecurityStatus/SecurityDescription_-2',
 -1: 'Notifications/SecurityStatus/SecurityDescription_-1',
 0: 'Notifications/SecurityStatus/SecurityDescription_0',
 1: 'Notifications/SecurityStatus/SecurityDescription_1',
 2: 'Notifications/SecurityStatus/SecurityDescription_2',
 3: 'Notifications/SecurityStatus/SecurityDescription_3',
 4: 'Notifications/SecurityStatus/SecurityDescription_4',
 5: 'Notifications/SecurityStatus/SecurityDescription_5',
 6: 'Notifications/SecurityStatus/SecurityDescription_6',
 7: 'Notifications/SecurityStatus/SecurityDescription_7',
 8: 'Notifications/SecurityStatus/SecurityDescription_8',
 9: 'Notifications/SecurityStatus/SecurityDescription_9',
 10: 'Notifications/SecurityStatus/SecurityDescription_10'}
rankLost = {const.factionCaldariState: 'UI/FactionWarfare/Ranks/RankLostCaldari',
 const.factionMinmatarRepublic: 'UI/FactionWarfare/Ranks/RankLostMinmatar',
 const.factionAmarrEmpire: 'UI/FactionWarfare/Ranks/RankLostAmarr',
 const.factionGallenteFederation: 'UI/FactionWarfare/Ranks/RankLostGallente'}
rankGain = {const.factionCaldariState: 'UI/FactionWarfare/Ranks/RankGainCaldari',
 const.factionMinmatarRepublic: 'UI/FactionWarfare/Ranks/RankGainMinmatar',
 const.factionAmarrEmpire: 'UI/FactionWarfare/Ranks/RankGainAmarr',
 const.factionGallenteFederation: 'UI/FactionWarfare/Ranks/RankGainGallente'}
notificationTypes = {'notificationTypeOldLscMessages': 1,
 'notificationTypeCharTerminationMsg': 2,
 'notificationTypeCharMedalMsg': 3,
 'notificationTypeAllMaintenanceBillMsg': 4,
 'notificationTypeAllWarDeclaredMsg': 5,
 'notificationTypeAllWarSurrenderMsg': 6,
 'notificationTypeAllWarRetractedMsg': 7,
 'notificationTypeAllWarInvalidatedMsg': 8,
 'notificationTypeCharBillMsg': 9,
 'notificationTypeCorpAllBillMsg': 10,
 'notificationTypeBillOutOfMoneyMsg': 11,
 'notificationTypeBillPaidCharMsg': 12,
 'notificationTypeBillPaidCorpAllMsg': 13,
 'notificationTypeBountyClaimMsg': 14,
 'notificationTypeCloneActivationMsg': 15,
 'notificationTypeCorpAppNewMsg': 16,
 'notificationTypeCorpAppRejectMsg': 17,
 'notificationTypeCorpAppAcceptMsg': 18,
 'notificationTypeCorpTaxChangeMsg': 19,
 'notificationTypeCorpNewsMsg': 20,
 'notificationTypeCharLeftCorpMsg': 21,
 'notificationTypeCorpNewCEOMsg': 22,
 'notificationTypeCorpDividendMsg': 23,
 'notificationTypeCorpVoteMsg': 25,
 'notificationTypeCorpVoteCEORevokedMsg': 26,
 'notificationTypeCorpWarDeclaredMsg': 27,
 'notificationTypeCorpWarFightingLegalMsg': 28,
 'notificationTypeCorpWarSurrenderMsg': 29,
 'notificationTypeCorpWarRetractedMsg': 30,
 'notificationTypeCorpWarInvalidatedMsg': 31,
 'notificationTypeContainerPasswordMsg': 32,
 'notificationTypeCustomsMsg': 33,
 'notificationTypeInsuranceFirstShipMsg': 34,
 'notificationTypeInsurancePayoutMsg': 35,
 'notificationTypeInsuranceInvalidatedMsg': 36,
 'notificationTypeSovAllClaimFailMsg': 37,
 'notificationTypeSovCorpClaimFailMsg': 38,
 'notificationTypeSovAllBillLateMsg': 39,
 'notificationTypeSovCorpBillLateMsg': 40,
 'notificationTypeSovAllClaimLostMsg': 41,
 'notificationTypeSovCorpClaimLostMsg': 42,
 'notificationTypeSovAllClaimAquiredMsg': 43,
 'notificationTypeSovCorpClaimAquiredMsg': 44,
 'notificationTypeAllAnchoringMsg': 45,
 'notificationTypeAllStructVulnerableMsg': 46,
 'notificationTypeAllStrucInvulnerableMsg': 47,
 'notificationTypeSovDisruptorMsg': 48,
 'notificationTypeCorpStructLostMsg': 49,
 'notificationTypeCorpOfficeExpirationMsg': 50,
 'notificationTypeCloneRevokedMsg1': 51,
 'notificationTypeCloneMovedMsg': 52,
 'notificationTypeCloneRevokedMsg2': 53,
 'notificationTypeInsuranceExpirationMsg': 54,
 'notificationTypeInsuranceIssuedMsg': 55,
 'notificationTypeJumpCloneDeletedMsg1': 56,
 'notificationTypeJumpCloneDeletedMsg2': 57,
 'notificationTypeFWCorpJoinMsg': 58,
 'notificationTypeFWCorpLeaveMsg': 59,
 'notificationTypeFWCorpKickMsg': 60,
 'notificationTypeFWCharKickMsg': 61,
 'notificationTypeFWCorpWarningMsg': 62,
 'notificationTypeFWCharWarningMsg': 63,
 'notificationTypeFWCharRankLossMsg': 64,
 'notificationTypeFWCharRankGainMsg': 65,
 'notificationTypeAgentMoveMsg': 66,
 'notificationTypeTransactionReversalMsg': 67,
 'notificationTypeReimbursementMsg': 68,
 'notificationTypeLocateCharMsg': 69,
 'notificationTypeResearchMissionAvailableMsg': 70,
 'notificationTypeMissionOfferExpirationMsg': 71,
 'notificationTypeMissionTimeoutMsg': 72,
 'notificationTypeStoryLineMissionAvailableMsg': 73,
 'notificationTypeTutorialMsg': 74,
 'notificationTypeTowerAlertMsg': 75,
 'notificationTypeTowerResourceAlertMsg': 76,
 'notificationTypeStationAggressionMsg1': 77,
 'notificationTypeStationStateChangeMsg': 78,
 'notificationTypeStationConquerMsg': 79,
 'notificationTypeStationAggressionMsg2': 80,
 'notificationTypeFacWarCorpJoinRequestMsg': 81,
 'notificationTypeFacWarCorpLeaveRequestMsg': 82,
 'notificationTypeFacWarCorpJoinWithdrawMsg': 83,
 'notificationTypeFacWarCorpLeaveWithdrawMsg': 84,
 'notificationTypeCorpLiquidationMsg': 85,
 'notificationTypeSovereigntyTCUDamageMsg': 86,
 'notificationTypeSovereigntySBUDamageMsg': 87,
 'notificationTypeSovereigntyIHDamageMsg': 88,
 'notificationTypeContactAdd': 89,
 'notificationTypeContactEdit': 90,
 'notificationTypeIncursionCompletedMsg': 91,
 'notificationTypeCorpKicked': 92,
 'notificationTypeOrbitalAttacked': 93,
 'notificationTypeOrbitalReinforced': 94,
 'notificationTypeOwnershipTransferred': 95,
 'notificationTypeFWAllianceWarningMsg': 96,
 'notificationTypeFWAllianceKickMsg': 97}
notifyIDs = util.KeyVal(notificationTypes)
groupUnread = 0
groupAgents = 1
groupBills = 2
groupCorp = 3
groupMisc = 4
groupOld = 5
groupSov = 6
groupStructures = 7
groupWar = 8
groupContacts = 9
groupTypes = {groupAgents: [notifyIDs.notificationTypeAgentMoveMsg,
               notifyIDs.notificationTypeLocateCharMsg,
               notifyIDs.notificationTypeResearchMissionAvailableMsg,
               notifyIDs.notificationTypeMissionOfferExpirationMsg,
               notifyIDs.notificationTypeMissionTimeoutMsg,
               notifyIDs.notificationTypeStoryLineMissionAvailableMsg,
               notifyIDs.notificationTypeTutorialMsg],
 groupBills: [notifyIDs.notificationTypeAllMaintenanceBillMsg,
              notifyIDs.notificationTypeCharBillMsg,
              notifyIDs.notificationTypeCorpAllBillMsg,
              notifyIDs.notificationTypeBillOutOfMoneyMsg,
              notifyIDs.notificationTypeBillPaidCharMsg,
              notifyIDs.notificationTypeBillPaidCorpAllMsg,
              notifyIDs.notificationTypeCorpOfficeExpirationMsg],
 groupContacts: [notifyIDs.notificationTypeContactAdd, notifyIDs.notificationTypeContactEdit],
 groupCorp: [notifyIDs.notificationTypeCharTerminationMsg,
             notifyIDs.notificationTypeCharMedalMsg,
             notifyIDs.notificationTypeCorpAppNewMsg,
             notifyIDs.notificationTypeCorpAppRejectMsg,
             notifyIDs.notificationTypeCorpAppAcceptMsg,
             notifyIDs.notificationTypeCorpTaxChangeMsg,
             notifyIDs.notificationTypeCorpNewsMsg,
             notifyIDs.notificationTypeCharLeftCorpMsg,
             notifyIDs.notificationTypeCorpNewCEOMsg,
             notifyIDs.notificationTypeCorpDividendMsg,
             notifyIDs.notificationTypeCorpVoteMsg,
             notifyIDs.notificationTypeCorpVoteCEORevokedMsg,
             notifyIDs.notificationTypeCorpLiquidationMsg,
             notifyIDs.notificationTypeCorpKicked],
 groupMisc: [notifyIDs.notificationTypeBountyClaimMsg,
             notifyIDs.notificationTypeCloneActivationMsg,
             notifyIDs.notificationTypeContainerPasswordMsg,
             notifyIDs.notificationTypeCustomsMsg,
             notifyIDs.notificationTypeInsuranceFirstShipMsg,
             notifyIDs.notificationTypeInsurancePayoutMsg,
             notifyIDs.notificationTypeInsuranceInvalidatedMsg,
             notifyIDs.notificationTypeCloneRevokedMsg1,
             notifyIDs.notificationTypeCloneMovedMsg,
             notifyIDs.notificationTypeCloneRevokedMsg2,
             notifyIDs.notificationTypeInsuranceExpirationMsg,
             notifyIDs.notificationTypeInsuranceIssuedMsg,
             notifyIDs.notificationTypeJumpCloneDeletedMsg1,
             notifyIDs.notificationTypeJumpCloneDeletedMsg2,
             notifyIDs.notificationTypeTransactionReversalMsg,
             notifyIDs.notificationTypeReimbursementMsg,
             notifyIDs.notificationTypeIncursionCompletedMsg],
 groupOld: [notifyIDs.notificationTypeOldLscMessages],
 groupSov: [notifyIDs.notificationTypeSovAllClaimFailMsg,
            notifyIDs.notificationTypeSovCorpClaimFailMsg,
            notifyIDs.notificationTypeSovAllBillLateMsg,
            notifyIDs.notificationTypeSovCorpBillLateMsg,
            notifyIDs.notificationTypeSovAllClaimLostMsg,
            notifyIDs.notificationTypeSovCorpClaimLostMsg,
            notifyIDs.notificationTypeSovAllClaimAquiredMsg,
            notifyIDs.notificationTypeSovCorpClaimAquiredMsg,
            notifyIDs.notificationTypeSovDisruptorMsg,
            notifyIDs.notificationTypeAllStructVulnerableMsg,
            notifyIDs.notificationTypeAllStrucInvulnerableMsg,
            notifyIDs.notificationTypeSovereigntyTCUDamageMsg,
            notifyIDs.notificationTypeSovereigntySBUDamageMsg,
            notifyIDs.notificationTypeSovereigntyIHDamageMsg],
 groupStructures: [notifyIDs.notificationTypeAllAnchoringMsg,
                   notifyIDs.notificationTypeCorpStructLostMsg,
                   notifyIDs.notificationTypeTowerAlertMsg,
                   notifyIDs.notificationTypeTowerResourceAlertMsg,
                   notifyIDs.notificationTypeStationAggressionMsg1,
                   notifyIDs.notificationTypeStationStateChangeMsg,
                   notifyIDs.notificationTypeStationConquerMsg,
                   notifyIDs.notificationTypeStationAggressionMsg2,
                   notifyIDs.notificationTypeOrbitalAttacked,
                   notifyIDs.notificationTypeOrbitalReinforced,
                   notifyIDs.notificationTypeOwnershipTransferred],
 groupWar: [notifyIDs.notificationTypeAllWarDeclaredMsg,
            notifyIDs.notificationTypeAllWarSurrenderMsg,
            notifyIDs.notificationTypeAllWarRetractedMsg,
            notifyIDs.notificationTypeAllWarInvalidatedMsg,
            notifyIDs.notificationTypeCorpWarDeclaredMsg,
            notifyIDs.notificationTypeCorpWarFightingLegalMsg,
            notifyIDs.notificationTypeCorpWarSurrenderMsg,
            notifyIDs.notificationTypeCorpWarRetractedMsg,
            notifyIDs.notificationTypeCorpWarInvalidatedMsg,
            notifyIDs.notificationTypeFWCorpJoinMsg,
            notifyIDs.notificationTypeFWCorpLeaveMsg,
            notifyIDs.notificationTypeFWCorpKickMsg,
            notifyIDs.notificationTypeFWCharKickMsg,
            notifyIDs.notificationTypeFWCorpWarningMsg,
            notifyIDs.notificationTypeFWCharWarningMsg,
            notifyIDs.notificationTypeFWCharRankLossMsg,
            notifyIDs.notificationTypeFWCharRankGainMsg,
            notifyIDs.notificationTypeFacWarCorpJoinRequestMsg,
            notifyIDs.notificationTypeFacWarCorpLeaveRequestMsg,
            notifyIDs.notificationTypeFacWarCorpJoinWithdrawMsg,
            notifyIDs.notificationTypeFacWarCorpLeaveWithdrawMsg,
            notifyIDs.notificationTypeFWAllianceWarningMsg,
            notifyIDs.notificationTypeFWAllianceKickMsg]}
                        */

#endif
